# Pitch Contour GUI - script for creating and editing piecewise linear curves
# composed of links and nodes. To be used along with the Tucker Maxon stuff
# Authors: Ritu Raj Tiwari (tiwari@technologist.com)
#          Johan Wouters
# Default values
package require Widgets
# The function calls at the bottom of file --->
# The initialization procedure. Set up everything
proc init {} {
 global ContourVars
 
 if [array exists ContourVars] {unset ContourVars}
 set ContourVars(allnames) "w bottom maxAmp maxTime state lastPointId  lastLineId nodeCount numCurves linkWidth linkColour nodeWidth nodeColour saveState saveList savePrompt" 
 foreach name $ContourVars(allnames) {
   if ![regexp Array $name] {
     set ContourVars($name) 0
   }
 }
 set ContourVars(nodeWidth) 3
 set ContourVars(linkWidth) 1
 set ContourVars(linkColour) blue
 set ContourVars(nodeColour) red
 set ContourVars(bgColour) #cdced6
 set ContourVars(state) 0
 set ContourVars(nodeCount) 0
 set ContourVars(fname) ""
 set ContourVars(saveState) "saved"
 set ContourVars(contoursFile) custom.contours
}

proc createToplevel {} {
 global ContourVars
 set w .pitchContour
 catch {destroy $w}
 toplevel $w
 lower .
 wm title $w "Pitch Contour"
 wm iconname $w "Curves"
 #positionWindow $w

 frame $w.mbar -relief raised -bd 1 ;# menubar on top
 pack $w.mbar -fill x
 set ContourVars(w) $w
 return $w
}

proc createContourCanvas {w bottom} {
 global ContourVars
 
 if ![array exists ContourVars] init
 set ContourVars(w) $w
 set ContourVars(bottom) $bottom
 frame $w.main  ;# the main frame
 canvas $w.main.c -width 410 -height 178 -bg $ContourVars(bgColour)
 #frame $bottom  ;# bottom frame...holds the co-ordinate field
 pack $w.main $w.main.c -fill both -expand 1
 set ContourVars(canvasWidth) [winfo reqwidth $w.main.c]
 set ContourVars(canvasHeight) [winfo reqheight $w.main.c]
} 
 
 # Now we setup the top menubar
proc topMenuBar {w} {
 menubutton $w.mbar.file -text File -underline 0 -menu $w.mbar.file.menu
 menubutton $w.mbar.edit -text Edit -underline 0 -menu $w.mbar.edit.menu
 pack $w.mbar.file $w.mbar.edit -side left
 	menu $w.mbar.file.menu -tearoff 0
 		$w.mbar.file.menu add command -label "Apply Pre-set Contour" -command {applyCustomContour 0}
 		$w.mbar.file.menu add command -label "Save Contour" -command {newCustomContour 0}
 		$w.mbar.file.menu add command -label "New Contours File" -command {newContoursFile }
 		$w.mbar.file.menu add command -label Exit -command {operateFile wrapup } 
 	menu $w.mbar.edit.menu -tearoff 0
 		$w.mbar.edit.menu add command -label "Set Background Colour" -command {setBgColour 1}
 		$w.mbar.edit.menu add cascade -label "Link Properties" -menu $w.mbar.edit.menu.link
 		$w.mbar.edit.menu add cascade -label "Node Properties" -menu $w.mbar.edit.menu.node
 		menu $w.mbar.edit.menu.link -tearoff 0
 			$w.mbar.edit.menu.link add command -label "Link Width" -command {setLinkWidth 0}
 			$w.mbar.edit.menu.link add command -label "Link Colour" -command {setLinkColour 0}
 		menu $w.mbar.edit.menu.node -tearoff 0
 			$w.mbar.edit.menu.node add command -label "Node Width" -command {setNodeWidth 0}
 			$w.mbar.edit.menu.node add command -label "Node Colour" -command {setNodeColour 0}
 $w configure -menu $w.mbar
}
 # Bottom frame - setup the co-ord field
proc bottomFrame {bottom} {
	global ContourVars
  
  label $bottom.minAmpLabel -text "Min: "
  entry $bottom.minAmp -width 4 -justify center
  label $bottom.maxAmpLabel -text "Max: "
  entry $bottom.maxAmp -width 4 -justify center
  label $bottom.coord -textvariable ContourVars(xy) 
  foreach w "$bottom.minAmp $bottom.maxAmp" {
    bind $w <KeyPress> {
     if {[string length "%A"]==1 && "%A">=" " && \
             "%A"<="~" && ("%A"<"0" || "%A">"9")} break
     if {"%A">="0" && "%A"<="9" && [string length [%W get]]>=4} break
    }
    bind $w <Tab> "processMinMaxEntry %W %A; break"
    bind $w <Return> "processMinMaxEntry %W %A; break"
  }
  #frame $bottom.save -width 10 -height 10 -bg #d9d2ca -bd 2 -relief raised
  pack $bottom.minAmpLabel $bottom.minAmp $bottom.maxAmpLabel $bottom.maxAmp \
    -side left -padx 2 -pady 3 -fill y
  pack $bottom.coord -side right -pady 5 
}

proc processMinMaxEntry {w a} {
  global ContourVars
  
  set top $ContourVars(w)
  set contour [extractContour]
  set which [lindex [split $w .] end]
  set ContourVars($which) [$w get]
  set contour [conversion $contour 0]
  $top.main.c delete all
  foreach name "nodeColourArray linkColourArray nodeWidthArray \
      linkWidthArray nodeX nodeY line2Id line1Id node1Id node2Id" {
    set names [array names ContourVars $name,*]
    foreach arrayName $names {
      unset ContourVars($arrayName)
    }
  }
  drawContour $contour
}  
 
proc processCanvasResize {w h} { 
  global ContourVars
  
  set contour [extractContour]
  set ContourVars(canvasWidth) $w
  set ContourVars(canvasHeight) $h
  set top $ContourVars(w)
  $top.main.c delete all
  foreach name "nodeColourArray linkColourArray nodeWidthArray \
      linkWidthArray nodeX nodeY line2Id line1Id node1Id node2Id" {
    set names [array names ContourVars $name,*]
    foreach arrayName $names {
      unset ContourVars($arrayName)
    }
  }
  set contour [conversion $contour 0]
  drawContour $contour
}  
  
 # About the main frame...
proc mainFrame {w} {
 global ContourVars
 
 bind $w.main.c <Configure> "processCanvasResize %w %h"
 bind $w.main.c <Motion> {showCoordinates %x %y}
 bind $w.main.c <1> {mouseDown %x %y}
 bind $w.main.c <ButtonRelease-1> {mouseUp %x %y}
 bind $w.main.c <B1-Motion> {middleDrag %x %y}
 bind $w.main.c <Button-3> {addNode %x %y}
 bind $w <Control-Button-1> {deleteNode %x %y}
 	# Bind the would be links and nodes to events
 	#$w.main.c bind node <B1-Motion> {mouseDrag %x %y}
 	#$w.main.c bind node <Motion> {setCurrentCurve %x %y}
 	#$w.main.c bind link <Button-3> {addNode %x %y}
 	#$w.main.c bind link <Motion> {setCurrentCurve %x %y}
}
 
 # Handle mouseDown event here
proc mouseDown {x y} {
 global ContourVars
 if {$ContourVars(state)==0} {
 		set $ContourVars(state) 1
 	}	
	return
}

# handle mouseDrag on nodes here
proc mouseDrag {x y} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	if {$state == 1} {
		set state 0
	} elseif {$state == 2 || $state == 3} {
		set saveState "modified"
		#$bottom.save configure -bg #ff4255 
		set state 3
		$w.main.c delete $lastLineId
		set lastLineId [$w.main.c create line $ContourVars(nodeX,$lastPointId) $ContourVars(nodeY,$lastPointId) $x $y -tag link -fill $linkColour -width $linkWidth]
		set ContourVars(linkColourArray,$lastLineId) $linkColour
		set ContourVars(linkWidthArray,$lastLineId) $linkWidth
		set ContourVars(line2Id,$lastPointId) $lastLineId
	}
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return
}	
	
# Handle mouseUp here
proc mouseUp {x y} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	if {$state == -1} {
		return
	} elseif {$state == 0} {
		return
	} elseif {$state == 1} {
		set state 2
		set saveState "modified"
		#$bottom.save configure -bg #ff4255
		set  new [$w.main.c create oval $x $y $x $y\
		-tag node -outline $nodeColour -width $nodeWidth]
		set ContourVars(nodeColourArray,$new) $nodeColour
		set ContourVars(nodeWidthArray,$new) $nodeWidth
		set ContourVars(nodeX,$new) $x
		set ContourVars(nodeY,$new) $y
		set ContourVars(line1Id,$new) 0
		set ContourVars(line2Id,$new) 0
		#set ContourVars(node1Id,$new) 0
		set lastPointId $new
		incr nodeCount
		set ContourVars(nodeArray,$nodeCount) $new
		set lastLineId 0
	} elseif {$state == 2} {
		return
	} else {
		set state 2
		set saveState "modified"
		$save configure -bg #ff4255
		set  new [$w.main.c create oval $x $y $x $y\
		 -tag node -outline $nodeColour -width $nodeWidth]
		set ContourVars(nodeColourArray,$new) $nodeColour
		set ContourVars(nodeWidthArray,$new) $nodeWidth
		set ContourVars(node1Id,$lastLineId) $lastPointId 
		set ContourVars(node2Id,$lastLineId) $new
		set ContourVars(line1Id,$new) $lastLineId
		set ContourVars(line2Id,$new) 0
		set ContourVars(line2Id,$lastPointId) $lastLineId	
		set ContourVars(nodeX,$new) $x;set ContourVars(nodeY,$new) $y
		#set ContourVars(node1Id,$currentCurve,$new) $lastPointId($currentCurve)
		set lastPointId $new
		set lastLineId 0;# This is the current last point
		incr nodeCount
		set ContourVars(nodeArray,$nodeCount) $new
	}
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return
}
# Handle right click - add point
proc addNode {x y} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	set saveState "modified"
	# $bottom.save configure -bg #ff4255
	# First find the ids of current link and neighbouring nodes
	set nnData [findNodes $x $y]
	set nodeToLeftId $ContourVars(nodeArray,[expr $nnData -1])
	set nodeToRightId $ContourVars(nodeArray,$nnData)
	set thisLinkId $ContourVars(line1Id,$nodeToRightId)
	# Draw a node here and set it up...
	set  new [$w.main.c create oval $x $y $x $y\
		-tag node -outline $nodeColour -width $nodeWidth]
	set ContourVars(nodeColourArray,$new) $nodeColour
	set ContourVars(nodeWidthArray,$new) $nodeWidth
	set ContourVars(nodeX,$new) $x; set ContourVars(nodeY,$new) $y
	# insert this node in the list
	for {set i $nodeCount } {$i >= $nnData} {incr i -1} {
		set ContourVars(nodeArray,[expr $i+1]) $ContourVars(nodeArray,$i)
	}
	set ContourVars(nodeArray,$nnData) $new
	incr nodeCount
	# Draw new links
  set thisNode1Id $ContourVars(node1Id,$thisLinkId)
  set newLeftLinkId [$w.main.c create line  $ContourVars(nodeX,$thisNode1Id) $ContourVars(nodeY,$thisNode1Id) $x $y -tag link$ -fill $linkColour -width $linkWidth]
	set ContourVars(linkColourArray,$newLeftLinkId) $linkColour
	set ContourVars(linkWidthArray,$newLeftLinkId) $linkWidth
	set newRightLinkId [$w.main.c create line $x $y $ContourVars(nodeX,$ContourVars(node2Id,$thisLinkId)) $ContourVars(nodeY,$ContourVars(node2Id,$thisLinkId)) -tag link -fill $linkColour -width $linkWidth]
	set ContourVars(linkColourArray,$newRightLinkId) $linkColour
	set ContourVars(linkWidthArray,$newRightLinkId) $linkWidth
	# Delete link between nodes
	$w.main.c delete  $thisLinkId
	# Setup the new links
	set ContourVars(line2Id,$nodeToLeftId) $newLeftLinkId
	set ContourVars(node1Id,$newLeftLinkId) $nodeToLeftId
	set ContourVars(line1Id,$new) $newLeftLinkId
	set ContourVars(node2Id,$newLeftLinkId) $new
	set ContourVars(line2Id,$new) $newRightLinkId
	set ContourVars(node1Id,$newRightLinkId) $new
	set ContourVars(node2Id,$newRightLinkId) $nodeToRightId
	set ContourVars(line1Id,$nodeToRightId) $newRightLinkId
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return
}
# Handle dragging of nodes
proc middleDrag {x y} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	set saveState "modified"
	#$bottom.save configure -bg #ff4255
	# Get to know your neighbours
	set nnData [getNearestNodeId $x $y]
	set i 0
	foreach element $nnData {
		incr i
		if {$i == 1} {
			set thisNodeId $element
		} else {
			set thisNodeIndex $element
		}
	}
	##puts "$thisNodeId $thisNodeIndex"
  set thisNodeColour $ContourVars(nodeColourArray,$thisNodeId)
	set thisNodeWidth $ContourVars(nodeWidthArray,$thisNodeId)
	if {$thisNodeIndex > 1} {
		set lineToLeftId $ContourVars(line1Id,$thisNodeId)
		set lineToLeftColour $ContourVars(linkColourArray,$lineToLeftId)
		set lineToLeftWidth $ContourVars(linkWidthArray,$lineToLeftId)
		set nodeToLeftId $ContourVars(node1Id,$lineToLeftId)
	}
	if {$thisNodeIndex < $nodeCount} {
		set lineToRightId $ContourVars(line2Id,$thisNodeId)
		set lineToRightColour $ContourVars(linkColourArray,$lineToRightId)
		set lineToRightWidth $ContourVars(linkWidthArray,$lineToRightId)
		set nodeToRightId $ContourVars(node2Id,$lineToRightId)
	}
	# Now delete this node and neighbouring lines
	$w.main.c delete $thisNodeId
	if {$thisNodeIndex > 1} {
		$w.main.c delete $lineToLeftId
	}
	if {$thisNodeIndex < $nodeCount} {
		$w.main.c delete $lineToRightId
	}
	# Now redraw the node and adjoining lines in updated position
	# PitchContour special - if this is the first or last node, no movement on the x-axis
	if { $thisNodeIndex == 1 || $thisNodeIndex == $nodeCount } {
		set x $ContourVars(nodeX,$thisNodeId)
	# even otherwise, MUST maintain functional form of curve, so a node cannot be dragged beyond its neighbours
	} elseif { $x < $ContourVars(nodeX,$ContourVars(nodeArray,[expr $thisNodeIndex - 1 ])) || $x > $ContourVars(nodeX,$ContourVars(nodeArray,[expr $thisNodeIndex + 1 ])) } {
			set x $ContourVars(nodeX,$thisNodeId)
	}
	# end PitchContour Special
	set  thisNodeId [$w.main.c create oval $x $y $x $y\
		-tag node -outline $thisNodeColour -width $thisNodeWidth]
	set ContourVars(nodeColourArray,$thisNodeId) $thisNodeColour
	set ContourVars(nodeWidthArray,$thisNodeId) $thisNodeWidth
	if {$thisNodeIndex > 1} {
		set lineToLeftId [$w.main.c create line $ContourVars(nodeX,$nodeToLeftId) $ContourVars(nodeY,$nodeToLeftId) $x $y -tag link -fill $lineToLeftColour -width $lineToLeftWidth]
		set ContourVars(linkColourArray,$lineToLeftId) $lineToLeftColour
		set ContourVars(linkWidthArray,$lineToLeftId) $lineToLeftWidth
	}
	if {$thisNodeIndex < $nodeCount } {
		set lineToRightId [$w.main.c create line $x $y $ContourVars(nodeX,$nodeToRightId) $ContourVars(nodeY,$nodeToRightId) -tag link -fill $lineToRightColour -width $lineToRightWidth]
		set ContourVars(linkColourArray,$lineToRightId) $lineToRightColour
		set ContourVars(linkWidthArray,$lineToRightId) $lineToRightWidth
	}
	# Now tell the neighbouring nodes about these new lines and node
	set ContourVars(nodeArray,$thisNodeIndex) $thisNodeId
	set ContourVars(nodeX,$thisNodeId) $x; set ContourVars(nodeY,$thisNodeId) $y
	if {$thisNodeIndex > 1} {
		set ContourVars(line1Id,$thisNodeId) $lineToLeftId
		set ContourVars(line2Id,$nodeToLeftId) $lineToLeftId
		set ContourVars(node2Id,$lineToLeftId) $thisNodeId
		set ContourVars(node1Id,$lineToLeftId) $nodeToLeftId
	}
	if {$thisNodeIndex < $nodeCount } {
		set ContourVars(line2Id,$thisNodeId) $lineToRightId
		set ContourVars(line1Id,$nodeToRightId) $lineToRightId
		set ContourVars(node1Id,$lineToRightId) $thisNodeId
		set ContourVars(node2Id,$lineToRightId) $nodeToRightId
	}
	if {$thisNodeIndex == $nodeCount} {set lastPointId $thisNodeId}
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return
}
# Handle deletion of nodes here
proc deleteNode {x y} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	set saveState "modified"
	#$bottom.save configure -bg #ff4255
	#puts "deleteNode called"
	# Get to know your node
	set nnData [getNearestNodeId $x $y]
	set i 0
	foreach element $nnData {
		incr i
		if {$i == 1} {
			set thisNodeId $element
		} else {
			set thisNodeIndex $element
		}
	}
	# Find out about your neighbours
	if {$thisNodeIndex > 1} {
		set lineToLeftId $ContourVars(line1Id,$thisNodeId)
		set nodeToLeftId $ContourVars(node1Id,$lineToLeftId)
	}
	if {$thisNodeIndex < $nodeCount} {
		set lineToRightId $ContourVars(line2Id,$thisNodeId)
		set nodeToRightId $ContourVars(node2Id,$lineToRightId)
	}
	# Now delete this node and neighbouring lines
	$w.main.c delete $thisNodeId
	if {$thisNodeIndex > 1} {
		$w.main.c delete $lineToLeftId
	}
	if {$thisNodeIndex < $nodeCount} {
		$w.main.c delete $lineToRightId
	}
	# If this was the first node, the erasing job is over...update nodeArray
	# 	to reflect new node numbers
	# If this was the last Node, update nodeArray
	# If this was somewhere in the middle, draw line between neighbours
	if {$nodeCount == 1} {
		set state -1
	} elseif {$thisNodeIndex == 1} {
		for {set i 2} {$i <= $nodeCount} {incr i} {
			set ContourVars(nodeArray,[expr $i-1]) $ContourVars(nodeArray,$i)
		}
	} elseif {$thisNodeIndex == $nodeCount} {
		#puts "deleted last node"
	} else {
		set newLineId [$w.main.c create line $ContourVars(nodeX,$nodeToLeftId) $ContourVars(nodeY,$nodeToLeftId) $ContourVars(nodeX,$nodeToRightId) $ContourVars(nodeY,$nodeToRightId) -tag link -fill $linkColour -width $linkWidth]
		set ContourVars(linkColourArray,$newLineId) $linkColour
		set ContourVars(linkWidthArray,$newLineId) $linkWidth
		# Tell the neighbouring nodes about the new line
		set ContourVars(line2Id,$nodeToLeftId) $newLineId
		set ContourVars(line1Id,$nodeToRightId) $newLineId
		# Tell the new line about neighbouring nodes
		set ContourVars(node1Id,$newLineId) $nodeToLeftId
		set ContourVars(node2Id,$newLineId) $nodeToRightId
		# Update nodeArray
		for {set i $thisNodeIndex} {$i < $nodeCount} {incr i} {
			set ContourVars(nodeArray,$i) $ContourVars(nodeArray,[expr $i+1])
		}
	}
	incr nodeCount -1
	if {$nodeCount != 0} {	
		set lastPointId $ContourVars(nodeArray,$nodeCount)
	} else {
		set state 0
	}
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return
}
			
# Return the ID of nearest point
proc getNearestNodeId {x y} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	set minDist 1000000
	set minDistId 1
	set minDistIndex 1
	for {set i 1} {$i<=$nodeCount} {incr i} {
		set nodeId $ContourVars(nodeArray,$i)
		set currentDist [distance $x $y $ContourVars(nodeX,$nodeId) $ContourVars(nodeY,$nodeId)]
		if {$currentDist < $minDist} {
			set minDist $currentDist
			set minDistId $ContourVars(nodeArray,$i)
			set minDistIndex $i
		}
	}	
	set retval "$minDistId $minDistIndex"
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return $retval
}
# Calculate distance between points
proc distance { x1 y1 x2 y2} {
	return [expr ($x1-$x2)*($x1-$x2) + ($y1-$y2)*($y1-$y2)]
}
# Find the neighbour nodes of a point (x,y) on a link 
proc findNodes {x y} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	# pitch Contour special - simplified func.
	# First find the nearest node
	set nnData [getNearestNodeId $x $y ]
	set nnId [lindex $nnData 0]
	set nnIndex [lindex $nnData 1]
	if { $x < $ContourVars(nodeX,$nnId) } { set retval $nnIndex } else { set retval [expr $nnIndex+1] }
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return $retval
	# end pitchContour special
}

# Set link width 
proc setLinkWidth {dummy} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	grab $w
	set d .dialog
	toplevel $d
	wm title $d "Set Link Width"
	label $d.l1 -text "Link Width:"
	entry $d.lw -width 2 -textvariable linkWidth
	pack $d.l1 $d.lw -side left
	bind $d <Return> "destroy $d"
	grab release $w
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return
}
proc setNodeWidth {dummy} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	grab $w
	set d .dialog
	toplevel $d
	wm title $d "Set Node Width"
	label $d.l1 -text "Node Width:"
	entry $d.lw -width 2 -textvariable nodeWidth
	pack $d.l1 $d.lw -side left
	bind $d <Return> "destroy $d"
	grab release $w
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return
}
proc setLinkColour {dummy} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	grab $w
	set dummy [tk_chooseColor -title "Choose Link Colour" -initialcolor $linkColour]
	if {[string compare $dummy ""] != 0} {
		set linkColour $dummy
	}
	grab release $w
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return
}
proc setNodeColour {dummy} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	grab $w
	set dummy [tk_chooseColor -title "Choose Node Colour" -initialcolor $nodeColour]
	if {[string compare $dummy ""] != 0} {
		set nodeColour $dummy
	}
	grab release $w
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return
}
proc setBgColour {dummy} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	grab $w
	set dummy [tk_chooseColor -title "Choose BackgroundColour" -initialcolor $bgColour]
	if {[string compare $dummy "" ] != 0} {
		set bgColour $dummy
	}
	$w.main.c configure -bg $bgColour
	grab release $w
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return
}

# Save the current canvas into a file
proc saveFile {dummy} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
  # Get the file name to save
  set types {
    {"Curve files"  {.curve} }
    {"All files"  *}
  }
  # Open the file
  if {[string compare $fname ""] == 0} {
    set fname [tk_getSaveFile -filetypes $types -parent $w -initialfile $fname]
    wm title $w "Curves - $fname"
  }
  set fid [open $fname w]
  # write the header
  puts $fid "curve file"
  puts $fid $bgColour
  # Loop through all the curves
  for {set i 1} {$i <= $numCurves} {incr i} {
  # See if this curve hasn't been deleted. Only then do the rest
    #if {[lsearch $deletedCurves $i] != -1} {
      # Now pick out nodes and links for this curve and write them  
        for {set j 1} {$j < $nodeCount($i) } {incr j} {
          set thisNodeId $ContourVars(nodeArray,$i,$j)
          set lineToRightId $ContourVars(line2Id,$i,$thisNodeId)
          set x $ContourVars(nodeX,$i,$thisNodeId)
          set y $ContourVars(nodeY,$i,$thisNodeId)
          set nColour $ContourVars(nodeColourArray,$thisNodeId)
          set nWidth $ContourVars(nodeWidthArray,$thisNodeId)
          set lColour $ContourVars(linkColourArray,$lineToRightId)
          set lWidth $ContourVars(linkWidthArray,$lineToRightId)
          puts $fid "$x $y $nColour $nWidth $lColour $lWidth"
        }
        # For the final node...no line to right of this node!
        set thisNodeId $ContourVars(nodeArray,$i,$nodeCount($i))
        set x $ContourVars(nodeX,$i,$thisNodeId)
        set y $ContourVars(nodeY,$i,$thisNodeId)
        set nColour $ContourVars(nodeColourArray,$thisNodeId)
        set nWidth $ContourVars(nodeWidthArray,$thisNodeId)
        puts $fid "$x $y $nColour $nWidth $lColour $lWidth"
        puts $fid "***"
    #}  
  }
  close $fid
  set saveState "saved"
  #$bottom.save configure -bg #d9d2ca 
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
  return
}

# Extract contour from the current canvas
proc extractContour {} {
  global ContourVars
  if [info exists ContourVars(nodeCount)] {
    set nodeCount $ContourVars(nodeCount)
    # in case mulitple curves: for {set i 1} {$i <= $numCurves} {incr i} 
    set i 1
    set contour ""
    # Now pick out nodes and links for this curve
    for {set j 1} {$j <= $nodeCount } {incr j} {
      set thisNodeId $ContourVars(nodeArray,$j)
      set x $ContourVars(nodeX,$thisNodeId)
      set y $ContourVars(nodeY,$thisNodeId)
      lappend contour $x $y
    }
    set contour [conversion $contour 1]
    set newContour ""
    foreach elem $contour {
      lappend newContour $elem
    }
    return $newContour
  } else {
    return ""
  }
}

#Open an existing curves file
proc openFile {dummy} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	destroy $w.main.c
	canvas $w.main.c -width 320 -height 240 -bg $bgColour
	pack $w.main.c
	bind $w.main.c <Motion> {set xy "%x, %y"}
	bind $w.main.c <1> {mouseDown %x %y}
	bind $w.main.c <ButtonRelease-1> {mouseUp %x %y}
	bind $w.main.c <B2-Motion> {middleDrag %x %y}
	set types {
		{"Curve files"	{curve} }
		{"All files"	*}
	}
	set fname [tk_getOpenFile -filetypes $types -parent $w ]
	set fid [open $fname r]
	set header [gets $fid]
	if {[string compare $header "curve file"] == 0} {
		set bgColour [gets $fid]
		$w.main.c configure -bg $bgColour
		set nodeCount 0
		set lastLineId 0
		# Bind the would be links and nodes to events
		$w.main.c bind node <B1-Motion> {mouseDrag %x %y}
		$w.main.c bind node <Control-Button-1> {deleteNode %x %y}
		$w.main.c bind node <Motion> {setCurrentCurve %x %y}
		$w.main.c bind link <Button-3> {addNode %x %y}
		$w.main.c bind link <Motion> {setCurrentCurve %x %y}
		# Now loop through the rest of the file
		set flag 0
		while { $flag == 0} {
			# Read the current node info
			set row [gets $fid]
			if { [string compare $row "" ] == 0 } {
				set flag 1
			} elseif { [string compare $row "***"] != 0 } {
			# if this isn't the last node of currentCurve...
				set x [lindex $row 0] ; set y [lindex $row 1]
				set nodeColour [lindex $row 2] ; set nodeWidth [lindex $row 3]
				set linkColour [lindex $row 4] ; set linkWidth [lindex $row 5]
				# Draw the current node and set it up
				puts $row
				set  new [$w.main.c create oval $x $y $x $y\
					-tag node -outline $nodeColour -width $nodeWidth]
				set ContourVars(nodeX,$new) $x ; set ContourVars(nodeY,$new) $y
				incr nodeCount
				set ContourVars(nodeArray,$nodeCount) $new
				set ContourVars(nodeColourArray,$new) $nodeColour 
				set ContourVars(nodeWidthArray,$new) $nodeWidth
				# if this isn't the first node, the last line is line1 for this node
				if { $nodeCount == 1 } {
					set lastPointId $new
				} else { 
					set lastLineId [$w.main.c create line $ContourVars(nodeX,$lastPointId) $ContourVars(nodeY,$lastPointId) $x $y -tag link -fill $linkColour -width $linkWidth]
					set ContourVars(linkColourArray,$lastLineId) $linkColour 
					set ContourVars(linkWidthArray,$lastLineId) $linkWidth
					set ContourVars(line1Id,$new) $lastLineId
					set ContourVars(line2Id,$lastPointId) $lastLineId
					set ContourVars(node1Id,$lastLineId) $lastPointId
					set ContourVars(node2Id,$lastLineId) $new
					set lastPointId $new
				}
				# now set the new colour and width parameters
			} else {
			# if this IS the last thing (***)
			#puts "***"
			set nodeCount 0
			set lastLineId 0
			# Bind the would be links and nodes to events
			$w.main.c bind node <B1-Motion> {mouseDrag %x %y}
			$w.main.c bind node <Control-Button-1> {deleteNode %x %y}
			$w.main.c bind node <Motion> {setCurrentCurve %x %y}
			$w.main.c bind link <Button-3> {addNode %x %y}
			$w.main.c bind link <Motion> {setCurrentCurve %x %y}
			}
		}
	} else {
		tk_messageBox -type ok -icon error -title "Curve: Error" -message "Your selection is not a valid curve file"
		set fname ""
	}		 
	close $fid
	set state 2
	incr currentCurve -1
	set numCurves $currentCurve
	wm title $w "Curves - $fname"
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return
}
# Handle new file here
proc operateFile {operation } {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	if { [string compare $saveState "modified"] == 0 } {
		set button [tk_messageBox -type yesnocancel -icon question -title "Curve: Question" -message "Do you want to save the current contour?"]
		if { [ string compare $button "yes" ] == 0 } {
			newCustomContour 0
			$operation 0
		} elseif { [ string compare $button "no"] == 0 } {
			$operation 0
		}
	} else {
		$operation 0	
	}
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return
} 
# Go ahead with destroying current canvas and starting new one
proc newFile {dummy} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	destroy $w.main.c
	canvas $w.main.c -width 320 -height 240 -bg $bgColour
	pack $w.main.c
	bind $w.main.c <Motion> {set xy "%x, %y"}
	bind $w.main.c <1> {mouseDown %x %y}
	bind $w.main.c <ButtonRelease-1> {mouseUp %x %y}
	bind $w.main.c <B2-Motion> {middleDrag %x %y}
	set numCurves 0
	set fname ""
	wm title $w "Curves - New"
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return
}
# Handle closing the window
proc wrapup { dummy} {
  global ContourVars
  set w $ContourVars(w)
	destroy $w
}
# Apply a pre-saved custom contour to this figure
proc applyCustomContour {dummy} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	set cName "" ; set cData ""
	set flag [getContoursFileName $contoursFile]
	if { $flag == 0 } {
	# So, if we have a valid contour file selected
	#  Get the name of the contour user wishes to impose...and the data corresponding to that name
		#puts "getting contour name..."
		set cName [getContourName $contoursFile 0]	
		#puts "contour name: $cName"
		#puts "getting contour data..."
		set cData [getContourData $contoursFile $cName]
	} else {
		set cData "50 100 300 100"
	}
	# First erase the existing canvas and start a blank one...
	$w.main.c delete node
	$w.main.c delete link
	# Now draw the contour 
	#puts "now drawing contour..."
	drawContour $cData
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
}
# Get name of contour
proc getContourName {cFile editability} {
	global ContourVars
  set cName $ContourVars(cName)
	# Get the list of names
	set nameList [getCList $cFile]
	# setup new dialog
	set d ".dialog"
	toplevel $d
	label $d.l1 -text "Choose a preset contour below and hit OK when done"
	combobox $d.c -textvariable cName -editable $editability
	foreach name $nameList	{ $d.c listinsert end $name }
	button $d.b1 -text OK -command "destroy $d"
	pack $d.l1 $d.c $d.b1
	#puts "waiting for $d to destoy"
	tkwait window $d
	#puts "destroyed"
  set ContourVars(cName) $cName
	return $cName
}
# Get list of contour names
proc getCList {cFile} {
	set fid [open $cFile r ]
	# Get list 
	while {[gets $fid tmp] >= 0} {
		lappend nameList $tmp
		gets $fid
	}
	close $fid
	return $nameList
}
# Return Contour data for the given contour name
proc getContourData {cFile cName} {
	set fid [open $cFile r ]
	while { [string compare [gets $fid] $cName ] !=0 } {}
	set retval [gets $fid]
	set retval [conversion $retval 0]
	close $fid
	return $retval
}

proc getContoursFileName {cfName} {
	global ContourVars
  set contoursFile $ContourVars(contoursFile)
	set flag 0
	# First check to see if the file exists
	if { [file exists $cfName ] == 0 } {
		set button [tk_messageBox -type yesno -icon error -title "Curve: Error" -message "No Custom Contours found in current directory. Do you want to provide an alternate path?"]
		if { [string compare $button "no" ] == 0 } {
			tk_messageBox -type ok -icon warning -title "Curve: Warning" -message "No pre-defined contours were loaded"
			set flag 1 
		} else {
			set contoursFile [tk_getOpenFile -filetypes {{"All files" *}} ]
			#tkwait variable contoursFile
			#puts "Contour File: $contoursFile "
			if {[string compare $contoursFile ""] == 0 } {
				tk_messageBox -type ok -icon warning -title "Curve: Warning" -message "No contours repository defined"
				#puts "blank file!!"
				set flag 1 
			}
		}
	}
  set ContourVars(contoursFile) $contoursFile
	return $flag
}
proc eraseContour {} {
  global ContourVars
  
  set ContourVars(nodeCount) 0
  if [info exists ContourVars(w)] { 
    if [winfo exists $ContourVars(w).main.c] {
      $ContourVars(w).main.c delete all 
    }
  }
  foreach name "nodeColourArray linkColourArray nodeWidthArray \
      linkWidthArray nodeX nodeY line2Id line1Id node1Id node2Id" {
    set names [array names ContourVars $name,*]
    foreach arrayName $names {
      unset ContourVars($arrayName)
    }
  }
}
  
proc drawContour {cData} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	set cName "Pitch Contour"
  set n 0
	set len [llength $cData]
	#puts "in drawContour..."
	for { set i 0 } { $i < $len } { incr i 2 } {
		# extract the co-ordinates of the current node
		set x [lindex $cData $i]
		set y [lindex $cData [expr $i + 1 ]]
		if { $i == 0 } {
			# Draw the current node
			set  new [$w.main.c create oval $x $y $x $y\
				-tag node -outline $nodeColour -width $nodeWidth]
			#puts "drawn node 1"
			# Now do the bookkeeping
			set ContourVars(nodeColourArray,$new) $nodeColour
			set ContourVars(nodeWidthArray,$new) $nodeWidth
			set lastPointId $new
			set ContourVars(nodeX,$new) $x
			set ContourVars(nodeY,$new) $y
			set ContourVars(line1Id,$new) 0
			set nodeCount 1
			set ContourVars(nodeArray,$nodeCount) $new
		} else {
			# Draw the current node
			set  new [$w.main.c create oval $x $y $x $y\
				-tag node -outline $nodeColour -width $nodeWidth]
			# Draw the line from lastPoint to this point
			set lastLineId [$w.main.c create line $ContourVars(nodeX,$lastPointId) $ContourVars(nodeY,$lastPointId) $x $y -fill $linkColour -width $linkWidth -tag link]
			# now the bookkeeping 
			set ContourVars(nodeColourArray,$new) $nodeColour
			set ContourVars(linkColourArray,$lastLineId) $linkColour
			set ContourVars(nodeWidthArray,$new) $nodeWidth
			set ContourVars(linkWidthArray,$lastLineId) $linkWidth
			set ContourVars(nodeX,$new) $x
			set ContourVars(nodeY,$new) $y
			set ContourVars(line2Id,$lastPointId) $lastLineId
			set ContourVars(line1Id,$new) $lastLineId
			set ContourVars(node1Id,$lastLineId) $lastPointId
			set ContourVars(node2Id,$lastLineId) $new
			set lastPointId $new
			incr nodeCount 
			#puts "drawn node $nodeCount"
			set ContourVars(nodeArray,$nodeCount) $new
		}
	}
	set state 2
	if {[winfo class $w] == "Toplevel"} {
    wm title $w "Pitch Contour - $cName"
  }
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
}
# Save the current contour as a new custom contour
proc newCustomContour {dummy} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	set flag 0
	# See if a contoursFile is defined
	if {[file exists $contoursFile] == 0} {
		set flag [getContoursFileName $contoursFile] 
	}
	# get the name of contour
	set cName [getNewContourName $contoursFile 1] 
	# If the sucker is fooling around...
	if {[string compare $cName "" ] == 0 } {
		tk_messageBox -type ok -icon warning -title "Curve: Warning" -message "Need a name for contour. Contour not saved"
		set flag 1
	}
	# if everything was fine
	if { $flag == 0 } {
		set cList [getCList $contoursFile ]
		# Two kinds of action here. If this is a new contour, simply append to ontoursFile. Else, modify previous entry
		if {[lsearch $cList $cName] == -1 } {
			appendContour 0 $cName
		} else {
			modifyContour $cName
		}
		wm title $w "Pitch Contour - $cName"
		set saveState "saved"
		#$bottom.save configure -bg #d9d2ca 
	}
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
}
# Get new contour name - also has the option to select from existing contours
proc getNewContourName {cFile editability} {
	global cName cType newCName
	# Get the list of names of existing contours
	set nameList [getCList $cFile]
	# setup new dialog
	set d ".dialog"
	toplevel $d
	label $d.msg -text "Specify a new contour name or choose from existing ones"
	radiobutton $d.rb1 -text "New Contour" -variable cType -value "New" 
	radiobutton $d.rb2 -text "use Existing Contour" -variable cType -value "Old" 
	entry $d.e -width 8 -textvariable newCName
	combobox $d.c -textvariable cName -editable $editability
	foreach name $nameList	{ $d.c listinsert end $name }
	button $d.b1 -text OK -command "destroy $d"
	pack $d.msg $d.rb1 $d.e $d.rb2 $d.c $d.b1
	#puts "waiting for $d to destoy"
	tkwait window $d
	#puts "destroyed"
	if {[string compare $cType "New" ] == 0} {
		set cName $newCName
	}
	return $cName
}
# Append to contoursFile
proc appendContour {outputFlag cName} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	if {$outputFlag == 0} {
		set fid [open $contoursFile a]
	} else {
		set fid ""
	}
	puts $fid  "\n$cName"
	for {set i 1} {$i <= $nodeCount} {incr i} {
		#puts -nonewline $fid "$ContourVars(nodeX,$ContourVars(nodeArray,$i)) $ContourVars(nodeY,$ContourVars(nodeArray,$i)) "
		lappend saveList "$ContourVars(nodeX,$ContourVars(nodeArray,$i)) $ContourVars(nodeY,$ContourVars(nodeArray,$i)) "
	}
	puts $fid [conversion $saveList 1]
	#puts $fid ""
	close $fid
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
}
# Modify existing contour
proc modifyContour {cName} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	# Read in the stuff from contoursFile and put it into a hash table
	set fid [open $contoursFile r]
	while {[gets $fid tmp] >= 0} {
		set cData [gets $fid]
		set cDataArray($tmp) $cData
	}
	close $fid
	# Now create cData from currently drawn contour
	set cData ""
	for {set i 1} {$i <= $nodeCount} {incr i} {
		#set cData $cData" $ContourVars(nodeX,$ContourVars(nodeArray,$i)) $ContourVars(nodeY,$ContourVars(nodeArray,$i))"
		lappend saveList "$ContourVars(nodeX,$ContourVars(nodeArray,$i)) $ContourVars(nodeY,$ContourVars(nodeArray,$i)) "
	}	
	# now set this stuff as the cData for the given contour name
	set cDataArray($cName) [conversion $saveList 1] 
	# get the list of names 
	set nameList [getCList $contoursFile]
	# Open the contoursFile for writing and write out the whole stuff
	set i 0
	while {[string compare [lindex $nameList $i] "" ] != 0 } { incr i}
	set len [expr $i -1]
	#puts " Length of nameList: $len"
	set i 0
	set fid [open $contoursFile w]
	foreach name $nameList {
	# if its the last line, don't write the newline character
		puts $fid "$name"
		foreach entry $cDataArray($name) {
			puts -nonewline $fid "$entry "
		}
		if {$i < $len} {puts -nonewline $fid "\n"}
		puts $i
		incr i
	}
	close $fid
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
}
# Define new contours file
proc newContoursFile {} {
	#puts "Reached newContoursFile func"
	if {[getContoursFileName ""] == 0 } {puts "Done"}
}
# Convert time/amplitude domain info into pixel values
# convFlag = 0 is to pixels. convFlag = 1 is from pixels.
proc conversion {contour convFlag args} {
  global ContourVars
  if {$contour == ""} { return $contour }
  if {$args != ""} {
    foreach {minTime maxTime minAmp maxAmp} $args break
  } elseif [info exists ContourVars(minTime)] {
    set minTime $ContourVars(minTime) 
    set maxTime $ContourVars(maxTime) 
    set minAmp $ContourVars(minAmp)
    set maxAmp $ContourVars(maxAmp)
  } else {
    foreach {minTime maxTime minAmp maxAmp} [getExtrema $contour] break
  }
  set ContourVars(minTime) $minTime
  set ContourVars(maxTime) $maxTime
  set ContourVars(minAmp) $minAmp
  set ContourVars(maxAmp) $maxAmp
	# Get the current width of the drawing canvas
	set width $ContourVars(canvasWidth)
	set height $ContourVars(canvasHeight)
	# get the max extent of time in the given pitchContour (args)
	if {$convFlag == 0 } {
		#foreach {maxTime maxAmp minAmp} [getMaxTime $args] break
    set ratio [expr $width/$maxTime]
	} else {
		set ratio [expr $maxTime/$width]
	}
	# Now scale every time/x -coordinate entry
	set flag 0
  set maxAmp [expr 1.0*($maxAmp-$minAmp)]
  set maxTime [expr 1.0*($maxTime - $minTime)]
  set retval ""
	foreach element $contour {
		# remove the braces if any
		regsub -all \{ $element "" element 
		regsub -all "\}" $element "" element 
		if {[string compare $element ""] != 0} {
    set element [expr 1.0 * $element]
    if {$flag == 0 } { 
			# if it is an even-numbered element - x coord
			if {$convFlag == 0 } {
        #set element [expr $width*(0.1+0.8*$element/$maxTime)]
        set element [expr $width*($element-$minTime)/$maxTime]
			} else {
        #set element [expr ($element/$width -0.1)*$maxTime/0.8]
        set element [expr $minTime+($element/$width)*$maxTime]
			}
			lappend retval $element
			set flag 1
		} else {
			# if it is an odd-numbered element - y coord
			if {$convFlag == 0 } {
				# Amplitude to y pixels
        set element [expr $height*(0.9 - 0.8*($element-$minAmp)/$maxAmp)]
        #set element [expr $height*(1.5-$element/$maxAmp)]
			} else {
				set element [expr $minAmp + ($maxAmp/0.8)*(0.9-$element/$height)]
			}
			lappend retval $element
			set flag 0
		}} else { puts "skipping this entry" }
	}
	return $retval
}
# get the max x coord from a given list
proc getExtrema { contour } {
  global ContourVars
  if {[llength $contour] > 1} {
    set minTime [lindex $contour 0]
    set minAmp [lindex $contour 1]
    set maxAmp $minAmp
    foreach {x y} $contour {
      if {$y < $minAmp} {
          set minAmp $y
      }
      if {$y > $maxAmp} {
        set maxAmp $y
      }
    }
    set maxTime $x
    # round off extrema to close integers (for display purpose)
    foreach val "minTime minAmp" {
      set $val [expr round([set $val]-0.5)]
    }
    foreach val "maxTime maxAmp" {
      set $val [expr round([set $val]+0.5)]
    }
  } else {
    foreach {minTime maxTime maxAmp minAmp} "0 0 0 0" break
  }
  set retval "$minTime $maxTime $maxAmp $minAmp"
  if [info exists ContourVars(bottom)] {
    set bottom $ContourVars(bottom)
    $bottom.minAmp delete 0 end
    $bottom.maxAmp delete 0 end
    $bottom.minAmp insert end $minAmp
    $bottom.maxAmp insert end $maxAmp
  }
  return $retval
}

# puts the current contour into a global variable
proc getCurrentContour {} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }
	for {set i 1} {$i <= $nodeCount} {incr i} {
		lappend saveList "$ContourVars(nodeX,$ContourVars(nodeArray,$i)) $ContourVars(nodeY,$ContourVars(nodeArray,$i)) "
	}	
  foreach name $ContourVars(allnames) {
    set ContourVars($name) [set $name]
  }
	return [conversion $saveList 1]
}

proc showCoordinates {x y} {
  global ContourVars
  
  foreach {x y}  [conversion [list $x $y] 1] break
  set x [format "%5.f" $x]
  set y [format "%4.f" $y]
  set text "Time: $x ms   Frequency: $y Hz"
  set ContourVars(xy) $text
}

# called by the Done! button
proc done {} {
  global ContourVars
  foreach name $ContourVars(allnames) {
    set $name $ContourVars($name)
  }  
	set currentContour [getCurrentContour]
	if {$savePrompt == 0} {
		set saveState "saved"
	}
  foreach name "savePrompt saveState currentContour" {
    set ContourVars($name) [set $name]
  }
  operateFile wrapup
}

# this proc shouldn't really be called as the targets timestamps from Festival 
# should be strictly ascending
proc sortContour {contour} {	
  set tmpContour {}
  foreach {a b} $contour { lappend tmpContour "$a $b" }
  set tmpContour [lsort -index 0 $tmpContour]
  set contour {}
  foreach a $tmpContour {
    eval "lappend contour $a"
  }
  return $contour
}
  
# for use from markup GUI  
proc initContour {w bottom} {
  init
  createContourCanvas $w $bottom
  bottomFrame $bottom
  mainFrame $w
}

proc showContour {contour PCstart PCend} {
  #set contour [sortContour $contour]
  foreach {d1 d2 maxAmp minAmp} [getExtrema $contour] break
  #foreach {x y} $contour { puts "DB: $x $y" }
  set contour [conversion $contour 0 $PCstart $PCend $minAmp $maxAmp]
  drawContour $contour
}

# all the function calls!!
#proc pitchContour {args} {
#  init
#  set w [createToplevel]
#  pack [frame $w.bottom]
#  createContourCanvas $w $w.bottom
#  topMenuBar $w
#  bottomFrame $w.bottom
#  mainFrame $w
#  drawContour [conversion $args 0 0 ]
#}
