proc markupParseTest {} {
  markupParseInit
	set text {<SABLE> yes <BLA/> yes  <BREAK/> <PITCH id="123"  id2=" bla "> yes </PITCH> yes </SABLE>}
	markupParse $text .t 
}

proc markupParseInit {} {
	global MarkupGui
	
	package require expat
	set parser [expat myparser]
	$parser configure -elementstartcommand elementStartCmd \
	                   -elementendcommand elementEndCmd \
	 									 -characterdatacommand dataCmd
	set MarkupGui(xmlParser) $parser
}

proc markupParse {text graphtext} {
  global MarkupGui
	
  if [winfo exists $graphtext] {
		$graphtext delete 1.0 end
		set t $graphtext
	} else {
	  set t [text $graphtext]
	}
  set MarkupGui(parseGraphtext) $t
  pack $t
	foreach element $MarkupGui(elementNames) {
		set MarkupGui(stack,$element) ""
	}
	set parser $MarkupGui(xmlParser)
										 
  $parser reset
	$parser parse $text
  if [winfo ismapped $graphtext] {
    propagateFontChanges $graphtext
  }
  # because sable needed space after opening <SABLE> tag, remove it here
  if {[$t get 1.0] == " "} { $t delete 1.0 }
}

# if known element, push on element stack, together with graphtext index
proc elementStartCmd {name attlist} {
	global MarkupGui
	
	set t $MarkupGui(parseGraphtext)
  if [isElement $name] {
		set indx [$t index end-1c]
		set MarkupGui(stack,$name) \
		  "{{$attlist} $indx} $MarkupGui(stack,$name)"
	} else {
		puts "unknown tag $name, leaving unparsed."
		$t insert end "<$name $attlist>"
	}
}
	
# if known element, pop from element stack, find index, apply tag
proc elementEndCmd {name} {
	global MarkupGui
	
	set t $MarkupGui(parseGraphtext)
  if [isElement $name] {
		if {[info exists MarkupGui(stack,$name)] & \
		    [llength $MarkupGui(stack,$name)] > 0} {
			foreach {attlist indx1} [lindex $MarkupGui(stack,$name) 0] break
			set attNames ""; set atts ""
			foreach {attName val} $attlist {
				lappend attNames $attName
				lappend atts [graphValue $name $attName $val]
			}
			set MarkupGui(stack,$name) [lrange $MarkupGui(stack,$name) 1 end]
			set indx2 [$t index end-1c]
			#puts "applyTag [list $name $attNames $atts] $t {$indx1 $indx2}" 
			applyTag [list $name $attNames $atts] $t "$indx1 $indx2" 
		} else {
			# should not occur if parser works correctly
			puts "syntax error: no start tag found for end tag $name"
		}
	} else {
		puts "unknown tag $name, leaving unparsed."
		$t insert end </$name>
	}
}
		
proc dataCmd {data} {
	global MarkupGui
	
	set t $MarkupGui(parseGraphtext)
	$t insert end "$data"
}
	
proc applyTag_test {tag graphtext highlight} {
	# for now, just create uniform tag
	eval "$graphtext tag add tag1 $highlight"
	$graphtext tag configure tag1 -foreground red
}

proc isElement {element} {
	global MarkupGui
	
	if {[lsearch $MarkupGui(elementNames) $element] > -1} {
		return 1
	} else {
		return 0
	}
}
